// bin2c.c
//
// convert a binary file into a C source vector
//
// put into the public domain by Sandro Sigala
// modified by Phillip Seaver
//
// syntax:  bin2c [-c] [-z] <input_file> <output_file>
//
//          -c    add the "const" keyword to definition
//          -z    terminate the array with a zero (useful for embedded C strings)
//
// examples:
//     bin2c -c myimage.png myimage_png.cpp
//     bin2c -z sometext.txt sometext_txt.cpp

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef PATH_MAX
#define PATH_MAX 1024
#endif

int useconst = 0;
int zeroterminated = 0;

int myfgetc(FILE *f)
{
	int c = fgetc(f);
	if (c == EOF && zeroterminated) {
		zeroterminated = 0;
		return 0;
	}
	return c;
}

void process(int count, const char *ifname_list[], const char *ofname)
{
	FILE *ifile, *ofile;
	char buf[PATH_MAX], *p;
	const char *cp;
	int c, col = 1;
	unsigned long nSize;

	ofile = fopen(ofname, "wb");
	if (ofile == NULL) {
		fprintf(stderr, "cannot open %s for writing\n", buf);
		exit(1);
	}

	while (count--)  {
		const char *ifname = *ifname_list++;
		ifile = fopen(ifname, "rb");
		if (ifile == NULL) {
			fprintf(stderr, "cannot open %s for reading\n", ifname);
			exit(1);
		}
		if ((cp = strrchr(ifname, '/')) != NULL)  {
			++cp;
		} else  {
			if ((cp = strrchr(ifname, '\\')) != NULL)  {
				++cp;
			} else  {
				cp = ifname;
			}
		}
		strcpy(buf, cp);
		for (p = buf; *p != '\0'; ++p)  {
			if (!isalnum(*p))  {
				*p = '_';
			}
		}

		if (fseek(ifile, 0, SEEK_END) != 0)  {
			fprintf(stderr, "cannot seek to end of %s\n", ifname);
			exit(1);
		}
		nSize = ftell(ifile);
		if (fseek(ifile, 0, SEEK_SET) != 0)  {
			fprintf(stderr, "cannot seek to beginning of %s\n", ifname);
			exit(1);
		}
		fprintf(ofile, "%sunsigned char %s[%lu] = {\n",
			useconst ? "const " : "", buf, nSize);
		while ((c = myfgetc(ifile)) != EOF) {
			if (col >= 78 - 6) {
				fputc('\n', ofile);
				col = 1;
			}
			fprintf(ofile, "0x%.2x, ", c);
			col += 6;

		}
		fprintf(ofile, "\n};\n");
		fclose(ifile);
	}

	fclose(ofile);
}

void usage(void)
{
	fprintf(stderr, "usage: bin2c [-cz] <output_file> <input_file ...>\n");
	exit(1);
}

int main(int argc, char **argv)
{
	while (argc > 1 && argv[1][0] == '-') {
		if (!strcmp(argv[1], "-c")) {
			useconst = 1;
			--argc;
			++argv;
		} else if (!strcmp(argv[1], "-z")) {
			zeroterminated = 1;
			--argc;
			++argv;
		} else if (!strcmp(argv[1], "--")) {
			--argc;
			++argv;
			break;
		} else {
			usage();
		}
	}
	if (argc < 3) {
		usage();
	}
	process(argc - 2, argv + 2, argv[1]);
	return 0;
}
